; SD/MMC RELATED DEFINES:

MMC_GO_IDLE_STATE       equ     $40
MMC_SEND_OP_COND        equ     $41
MMC_READ_CID            equ     $4A
MMC_SET_BLOCK_SIZE      equ     $50
MMC_READ_SINGLE_BLOCK   equ     $51
MMC_READ_MULTIPLE_BLOCK equ     $52
MCC_TERMINATE_MULTI_READ equ    $4C
MMC_WRITE_SINGLE_BLOCK  equ     $58
MMC_WRITE_MULTIPLE_BLOCK equ    $59
MMC_STOP_TRAN           equ     $FD


SPI_PORT        equ     $10

BLOCKSIZE       equ     512     ; SD/MMC block size (bytes)

FATSIZE         equ     8       ; dimensioni FAT in termini di blocchi da 64KBytes (in pratica e` l'offset
                                ; da caricare nella word MSB dell'indirizzo della SD per accedere al primo
                                ; cluster di dati, subito oltre la FAT). 8 = 512KBytes (8192 entries da 64 bytes)

MMCSIZE         equ     8192    ; dimensioni della MMC in termini di blocchi da 128K: 8192 = 1GB
                                ; Prima di prendere/ripristinare uno snapshot, il programmma verifica che l'ubicazione
                                ; dei dati non sia oltre il limite fisico, tenendo anche conto dello spazio occupato
                                ; dalla FAT. Con una FATSIZE = 8 (4 blocchi da 128K) e una MMC da 1GB (8192 blocchi)
                                ; sara` possibile gestire 8192-4 = 8188 snapshots.

OFFSET_GRAN     equ     16      ; sd_check ora cerca l'header snapshot a piu' offset, spaziati di OFFSET_GRAN.
                                ; con 16 avremo salti di 256MB (per cui controlla a 0, +256, +512, etc
        call    cs_high                 ; set cs high

MAXOFFSET       equ     112     ; limite ricerca (compreso): con 48 cerca fino a +768MB, con 96 fino a +1,5GB

cs_high:
	ld	a,6	; CS = 1, DATA = 1, CLK = 0
	out	(SPI_PORT),a
	ret

cs_low:
	ld	a,2	; CS = 0, DATA = 1, CLK = 0
	out	(SPI_PORT),a
	ret


mmc_wait_response:
	push	bc
	ld	b,8	; will read 8 bits
	xor	a	; Initially we have 0
	ld	c,a	; we will keep it to c

mmc_wait_response_read_bit:

	ld	a,c	; c = c*2 (or can we rotate C left?)
	add	a,a
	ld	c,a

	ld	a,3	; CS= 0, data=1, clock=1
	out 	(SPI_PORT),a ;Should be read at Falling edge, but this is a rising one?
	in	a,(SPI_PORT)
	push	af
	call	outbytehex
	pop	af
	and	1	; only	lower bit is significant
	xor	1	; our input is inverted
	add	a,c	
	ld	c,a

	ld	a,2	; CS=0, data=1,clock=0
	out     (SPI_PORT),a
	djnz	mmc_wait_response_read_bit

	ld	a,c	; helding result in a
	push	af
	call	outstringinline
	db	"Read byte: ",0
	pop	af
	push	af
	call	outbytehex
	call	outstringinline
	db	$0D,$0A,0
	pop	af
	
	pop	bc
	ret

;  Sends a command with parameters = 00 and checksum = $95. Destroys AF.
; Called from initmmc:
mmc_write_command:
	push	af
	call	outstringinline
	db	"Writing Command: ",0
	pop	af
	push	af
	call	outbytehex
	call	outstringinline
	db $0D,$0A,0
	pop	af
	call	mmc_write_byte
	xor	a
	call	mmc_write_byte
	xor	a
	call	mmc_write_byte
	xor	a
	call	mmc_write_byte
	xor	a
	call	mmc_write_byte
	ld	a,$95			; $95 is only needed when the CARD INIT is being performed			
	call	mmc_write_byte		; then this byte is ignored.
	ret
	
; Called by mmc_write_command
mmc_write_byte:
	push	bc
	push	af
	call	outstringinline
	db	"WB: ",0
	pop	af
	push	af
	call	outbytehex
	pop	af
	ld	b,8
mmc_write_command_bit_loop:
;	push	af
;	ld	a,2
;	out	(SPI_PORT),a	; CS 0 Data 1 Clk 0 , Falling edge  /*** THINK ***/
;	pop	af
	add	a,a
	push	af
	ld	a,0		; will XOR clear C flag?
	adc	a,0		; will add C = 1 for bit 1 and 0 for bit 0
	add	a,a		; if was c -> a==2
	out     (SPI_PORT),a   ; Data on data line, Clock low
	inc	a		; Clk to high
	out 	(SPI_PORT),a	; 
	push	af
	call	outbytehex
	pop	af
        dec	a		; clock to low, data still on data bus, but noone cares! 
	out 	(SPI_PORT),a
	pop	af
	djnz	mmc_write_command_bit_loop
	call	outstringinline
	db	"..Done",$0A,$0D,0
	pop	bc
;	ld	a,3		; CS-0,Data=1,Clock = 1
;	out     (SPI_PORT),a	; That seems to be wrong, we should keep clock = 0
	ret

mmc_send_ticks_cs_high:
	push	bc
	ld	a,7
	out 	(SPI_PORT),a	; cs-high,data-high,clock-up
	ld	a,'^'
	rst	8
	ld	a,6
	out     (SPI_PORT),a	; cs-high, data-high, clock-down
	ld	a,'v'
	rst	8
	pop	bc
	djnz	mmc_send_ticks_cs_high
	ret

initmmc:      
	call	outstringinline
	db	"Initing MMC",$0D,$0A,0
        call   cs_high                 ; set cs high
	ld	b,80
	call	mmc_send_ticks_cs_high
	call	cs_low
	ld	a,MMC_GO_IDLE_STATE
	call	mmc_write_command
	call	outstringinline
	db	"Command written",$0A,$0D,0
	ld	b,16		; wait for 01 response for 16 byte ticks - should be more then enought
initmmc_wait_init:
	call	mmc_wait_response
	dec	a
	jp	z,initmmc_init_done	; got 01 response, cool!
	djnz	initmmc_wait_init
	call	outstringinline
	db	"Unable to init MMC",$0A,$0D,0
	jp	initmmc_exit
initmmc_init_done:
	call	outstringinline
	db	"**Init done**",$0A,$0D,0
initmmc_exit:
	ret
